@@ -7,6 +7,8 @@ mod utils;
 #[cfg(feature = "clap")]
 pub mod cli;
 
+pub use api::*;
+pub use config::*;
 pub use entities::*;
 pub use errors::*;
 pub use mediaflow_derive::*;
